/* eslint-disable no-console */
import {
  Modal, ModalService, Button, TextInput, DialogService,
  AlertService, Text, Checkbox, Textarea, Date as ZdDate, Form, Time,
} from '@zeedhi/common';
import {
  Metadata, Http, IDictionary, Singleton,
} from '@zeedhi/core';
import {
  IterableComponentRender,
} from '@zeedhi/teknisa-components-common';

@Singleton
export class ApontamentoController {
  private modalEntry!: Modal;

  private modalExit!: Modal;

  private modalValidate!: Modal;

  private NRAPONTAMENTO!: number;

  private LASTDATEENTRY!: string;

  private NRFUNCIONARIO = localStorage.getItem('UserLogado');

  public async createModal() {
    this.modalEntry = await ModalService.createFromJson('Modais/modal-registrar-entrada', true);
    this.modalExit = await ModalService.createFromJson('Modais/modal-registrar-saida', true);
    this.modalValidate = await ModalService.createFromJson('Modais/modal-validar-saida', true);
  }

  public destroyModal() {
    this.modalValidate?.destroy();
    this.modalEntry?.destroy();
    this.modalExit?.destroy();
  }

  public openModalEntry() {
    this.modalEntry.show();
  }

  public openModalExit() {
    this.modalExit.show();
  }

  public openModalValidate() {
    this.modalValidate.show();
  }

  public closeModalValidate() {
    this.modalValidate.hide();
  }

  public closeModalEntry() {
    this.modalEntry.hide();
  }

  public closeModalExit() {
    this.modalExit.hide();
  }

  public openModalAppointment({ component }:{ component:Button }) {
    if (component.label === 'Entrada') {
      this.openModalEntry();
    } else {
      this.openModalExit();
    }
  }

  public setButtonRegister() {
    const NRFUNC = localStorage.getItem('UserLogado');

    Http.post('/typesRegister', { NRFUNC }).then((response: IDictionary) => {
      response.data.dataset.types.forEach((data: IDictionary) => {
        if (data.IDTIPORESGISTRO === 'E') {
          Metadata.getInstance<Button>('homeAppointment').label = 'Saída';
        } else {
          Metadata.getInstance<Button>('homeAppointment').label = 'Entrada';
        }
        this.NRAPONTAMENTO = data.NRAPONTAMENTO;
        this.LASTDATEENTRY = data.DTHRENTRADA;
      });
    });
  }

  public getCurrentDate() {
    const date = new Date();

    const hour = date.getHours();
    const min = date.getMinutes();

    const dateNow = `${date.toLocaleDateString()} ${hour}:${min}`;
    return dateNow;
  }

  public setCurrentDate() {
    const timeApp = this.getCurrentDate().split(' ')[1];
    const shortDate = this.getCurrentDate().split(' ')[0];
    Metadata.getInstance<Time>('input-time-appointment').value = timeApp;
    Metadata.getInstance<ZdDate>('date-appointment').value = shortDate;
    Metadata.getInstance<Text>('text-appointment-date-now').text = `Apontamento do dia: ${shortDate}`;
  }

  public registerAbsence() {
    const dateOfAbsence = Metadata.getInstance<ZdDate>('selectDate').value;
    const justify = Metadata.getInstance<Textarea>('input-justify-appointment').value;
    const form = Metadata.getInstance<Form>('form-appointment-absence');

    if (form.validate()) {
      Http.post('/registerAbsence', {
        DTAPONTAMENTO: dateOfAbsence,
        IDFALTA: 'S',
        DSOBSREGISTRO: justify,
        NRFUNCIONARIO: this.NRFUNCIONARIO,
      }).then((response: IDictionary) => {
        const message = response.data.dataset.absence[0];
        this.messages(message);
      });
    } else {
      AlertService.show({
        name: 'basic-alert',
        text: 'Por favor, preencha todos os campos!',
        top: true,
        left: false,
        timeout: 3000,
        color: 'error',
      });
    }
  }

  public registerByAppointment() {
    const dateAppointment = Metadata.getInstance<TextInput>('date-appointment').value;
    const timeAppointment = Metadata.getInstance<Time>('input-time-appointment').value;
    const dateAndTimeApp = `${dateAppointment} ${timeAppointment}`;
    const observation = Metadata.getInstance<Textarea>('input-observation-appointment').value;
    // console.log(dateAndTimeApp);

    Http.post('/typesRegister', { NRFUNC: this.NRFUNCIONARIO }).then((response: IDictionary) => {
      response.data.dataset.types.forEach((data: IDictionary) => {
        if (data.IDTIPORESGISTRO === 'E') {
          if (dateAppointment !== this.LASTDATEENTRY) {
            Promise.all([this.openModalValidate()]).then(() => {
              this.setDateModal();
            });
          } else {
            Http.post('/setNewRegisterAppointmentByButton', {
              CURRENTTIME: timeAppointment,
              DTHRENTRADA: null,
              DTAPONTAMENTO: dateAppointment,
              NRFUNCIONARIO: this.NRFUNCIONARIO,
              DSOBSREGISTRO: observation,
              IDTIPORESGISTRO: 'S',
              DTHRSAIDA: dateAndTimeApp,
              IDALMOCO: (Metadata.getInstance<Checkbox>('checkbox-lunch').value) ? 'S' : 'N',
            }).then((ret: IDictionary) => {
              const message = ret.data.dataset.app[0];
              this.messages(message);
            });
          }
        } else {
          Http.post('/setNewRegisterAppointmentByButton', {
            CURRENTTIME: timeAppointment,
            DTHRENTRADA: dateAndTimeApp,
            DTAPONTAMENTO: dateAppointment,
            NRFUNCIONARIO: this.NRFUNCIONARIO,
            DSOBSREGISTRO: observation,
            IDTIPORESGISTRO: 'E',
            DTHRSAIDA: null,
            IDALMOCO: 'N',
          }).then((ret: IDictionary) => {
            const message = ret.data.dataset.app[0];
            this.messages(message);
          });
        }
      });
    });
  }

  public registerExitLastDate(currentDate: string) {
    const currentTime = this.getCurrentDate().split(' ')[1];

    Http.post('/setLastAppointment', {
      CURRENTTIME: currentTime,
      DTHRENTRADA: null,
      DTAPONTAMENTO: this.LASTDATEENTRY,
      NRAPONTAMENTO: this.NRAPONTAMENTO,
      NRFUNCIONARIO: this.NRFUNCIONARIO,
      DSOBSREGISTRO: null,
      IDTIPORESGISTRO: 'S',
      DTHRSAIDA: currentDate,
      IDALMOCO: 'N',
    }).then((response: IDictionary) => {
      const message = response.data.dataset.last[0];
      this.messages(message);
    });
  }

  public registerExitDateToday(dateExit: string) {
    const currentTime = this.getCurrentDate().split(' ')[1];
    const NRFUNC = localStorage.getItem('UserLogado');

    Http.post('/setNewRegisterAppointmentByButton', {
      CURRENTTIME: currentTime,
      DTHRENTRADA: null,
      NRFUNCIONARIO: NRFUNC,
      DTAPONTAMENTO: dateExit,
      DSOBSREGISTRO: null,
      IDTIPORESGISTRO: 'S',
      DTHRSAIDA: dateExit,
      IDALMOCO: 'N',
    }).then((res: IDictionary) => {
      const message = res.data.dataset.app[0];
      this.messages(message);
    });
  }

  public setDateModal() {
    const buttonLastDate = Metadata.getInstance<Button>('button-register-lastDate');
    buttonLastDate.label = `Registrar: ${this.LASTDATEENTRY}`;
  }

  public registerExitByValidateModal({ component }:{ component: Button }) {
    const currentDate = this.getCurrentDate();

    if (component.name === 'button-register-today') {
      this.registerExitDateToday(currentDate);
    } else if (component.name === 'button-register-lastDate') {
      this.registerExitLastDate(this.LASTDATEENTRY);
    }
  }

  public registerEntryOrExitByModal({ component }:{ component: Button }) {
    const currentDate = this.getCurrentDate();
    const currentTime = this.getCurrentDate().split(' ')[1];
    const NRFUNC = localStorage.getItem('UserLogado');

    if (String(currentDate.split(' ')[0]) !== this.LASTDATEENTRY && component.name !== 'button-register-entry') {
      Promise.all([this.openModalValidate()]).then(() => {
        this.setDateModal();
      });
    } else {
      Http.post('/setNewRegisterAppointmentByButton', {
        CURRENTTIME: currentTime,
        DTAPONTAMENTO: null,
        DTHRENTRADA: (component.name === 'button-register-entry') ? currentDate : null,
        NRFUNCIONARIO: NRFUNC,
        DSOBSREGISTRO: null,
        IDTIPORESGISTRO: (component.name === 'button-register-exit' || component.name === 'button-register-lunch') ? 'S' : 'E',
        DTHRSAIDA: (component.name === 'button-register-exit' || component.name === 'button-register-lunch') ? currentDate : null,
        IDALMOCO: (component.name === 'button-register-lunch') ? 'S' : 'N',
      }).then((response: IDictionary) => {
        const message = response.data.dataset.app[0];
        this.messages(message);
      });
    }
  }

  public optionLunchDisable({ component }:{ component: Checkbox }) {
    Http.post('/optionLunch', { NRFUNC: this.NRFUNCIONARIO }).then((response: IDictionary) => {
      response.data.dataset.option.forEach(() => {
        const id = response.data.dataset.option[0][1][0];
        const qtd = response.data.dataset.option[0][0][0];
        if (id.IDTIPORESGISTRO !== 'E') {
          component.disabled = true;
        }
        if (qtd.QTDALMOCO === 1) {
          component.disabled = true;
          Metadata.getInstance<Button>('button-register-lunch').disabled = true;
        }
      });
    });
  }

  public messages(returnMessage: string) {
    if (returnMessage === 'OK') {
      AlertService.show({
        name: 'basic-alert',
        text: 'Apontamento realizado com sucesso!',
        top: true,
        left: false,
        timeout: 3000,
        color: 'success',
      });
      setTimeout(() => {
        window.location.reload();
      }, 2000);
    } else {
      DialogService.show({
        name: 'warning-dialog',
        type: 'warning',
        title: 'Erro!',
        text: 'Operação bloqueada! Este apontamento não pôde ser registrado.<br><br>Se o problema persistir entre em contato com o suporte.',
      });
    }
  }

  public cardsRegisters() {
    Http.post('/cardsRegisters', { NRFUNC: this.NRFUNCIONARIO }).then((response: IDictionary) => {
      response.data.dataset.cards.forEach((data: IDictionary) => {
        Metadata.getInstance<IterableComponentRender>('iCardsAppointment').datasource.data.push({
          NR: data.NRREGISTROHORAS,
          NMAPONTAMENTO: (data.IDALMOCO === 'S') ? `${data.IDTIPORESGISTRO} (Almoço)` : data.IDTIPORESGISTRO,
          HRAPONTAMENTO: (data.IDTIPORESGISTRO === 'Entrada') ? data.DTHRENTRADA : data.DTHRSAIDA,
        });
      });
    });
  }
}
